@@ -1,5 +1,10 @@
 #![cfg_attr(not(feature = "std"), no_std)]
 
-pub mod hashing_utils;
 // TODO: Feature gate this
+#[macro_use]
 pub mod serde_utils;
+pub mod ec;
+pub mod ff;
+pub mod hashing_utils;
+pub mod msm;
+pub mod poly;
