@@ -117,3 +117,54 @@ where
 {
     serializer.serialize_str(&error.to_string())
 }
+
+#[macro_export]
+macro_rules! impl_for_groth16_struct {
+    ($serializer_name: ident, $struct_name: ident, $error_msg: expr) => {
+        pub struct $serializer_name;
+
+        impl<E: PairingEngine> SerializeAs<$struct_name<E>> for $serializer_name {
+            fn serialize_as<S>(elem: &$struct_name<E>, serializer: S) -> Result<S::Ok, S::Error>
+            where
+                S: Serializer,
+            {
+                let mut bytes = vec![];
+                CanonicalSerialize::serialize(elem, &mut bytes)
+                    .map_err(serde::ser::Error::custom)?;
+                serializer.serialize_bytes(&bytes)
+            }
+        }
+
+        impl<'de, E: PairingEngine> DeserializeAs<'de, $struct_name<E>> for $serializer_name {
+            fn deserialize_as<D>(deserializer: D) -> Result<$struct_name<E>, D::Error>
+            where
+                D: Deserializer<'de>,
+            {
+                struct PVisitor<E: PairingEngine>(PhantomData<E>);
+
+                impl<'a, E: PairingEngine> Visitor<'a> for PVisitor<E> {
+                    type Value = $struct_name<E>;
+
+                    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                        formatter.write_str($error_msg)
+                    }
+
+                    fn visit_seq<A>(self, mut seq: A) -> Result<Self::Value, A::Error>
+                    where
+                        A: SeqAccess<'a>,
+                    {
+                        let mut bytes = Vec::<u8>::new();
+                        while let Some(b) = seq.next_element()? {
+                            bytes.push(b);
+                        }
+                        let p: $struct_name<E> =
+                            CanonicalDeserialize::deserialize(bytes.as_slice())
+                                .map_err(serde::de::Error::custom)?;
+                        Ok(p)
+                    }
+                }
+                deserializer.deserialize_seq(PVisitor::<E>(PhantomData))
+            }
+        }
+    };
+}
